/***************************************************************************

  glabel.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include <gtk/gtk.h>
#include "widgets.h"
#include "widgets_private.h"
#include <string.h>
#include <stdlib.h>

void gLabel::fontModified()
{

	return;
	PangoAttrList *pal=pango_attr_list_new();
	PangoAttribute *pa=pango_attr_strikethrough_new(true);
	pa->start_index = 0;
	pa->end_index = g_utf8_strlen(gtk_label_get_text(GTK_LABEL(widget)), -1); 
	pango_attr_list_insert(pal, pa);
	gtk_label_set_attributes(GTK_LABEL(widget), pal);
	pango_attr_list_unref(pal);
}



gLabel::gLabel(gControl *parent) : gControl(parent)
{
	GtkWidget *fr;	

	g_typ=Type_gLabel;
	border=gtk_event_box_new();
	widget=gtk_label_new("");
	fr=gtk_frame_new("");
	gtk_frame_set_shadow_type(GTK_FRAME(fr),GTK_SHADOW_NONE);
	gtk_frame_set_label_widget(GTK_FRAME(fr),NULL);
	#ifndef GTK_2_4
	gtk_label_set_single_line_mode (GTK_LABEL(widget),true);
	#endif
	gtk_label_set_line_wrap (GTK_LABEL(widget),true);
	gtk_label_set_justify(GTK_LABEL(widget),GTK_JUSTIFY_CENTER);
	gtk_label_set_use_markup (GTK_LABEL(widget),false);
	gtk_container_add (GTK_CONTAINER(fr),widget);
	gtk_container_add (GTK_CONTAINER(border),fr);	
	connectParent();
	initSignals();
	g_signal_connect(G_OBJECT(border),"button-press-event",G_CALLBACK(sg_button_Press),(gpointer)this);
	g_signal_connect_after(G_OBJECT(border),"motion-notify-event",G_CALLBACK(sg_motion),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"button-release-event",G_CALLBACK(sg_button_Release),(gpointer)this);
	setAlignment(alignNormal);
}

gTextLabel::gTextLabel(gControl *parent) : gLabel(parent)
{
	g_typ=Type_gTextLabel;
	#ifndef GTK_2_4
	gtk_label_set_single_line_mode (GTK_LABEL(widget),false);
	#endif
	gtk_label_set_use_markup (GTK_LABEL(widget),true);
	setAlignment(alignTopNormal);
}

int gLabel::alignment()
{
	gfloat x,y;
	int retval=0;
	
	gtk_misc_get_alignment (GTK_MISC(widget),&x,&y);

	if (!x) retval+=1; 
	else if (x==1) retval+=2; 
	
	if (!y) retval+=16; 
	else if (y==1) retval+=32;
	else retval+=64;
	
	return retval;
}

const char* gLabel::text()
{
	return gtk_label_get_text(GTK_LABEL(widget));
}

const char* gTextLabel::text()
{
	return gtk_label_get_text(GTK_LABEL(widget));
}

void gLabel::setAlignment(int al)
{
	gfloat x,y;
		
	switch (al)
	{
		case alignBottom: 	y=1; 	x=0.5; break;
		case alignBottomLeft: 	y=1; 	x=0; break;
		case alignBottomNormal: y=1; 	x=0; break;
		case alignBottomRight: 	y=1; 	x=1; break;
		case alignCenter: 	y=0.5; 	x=0.5; break;
		case alignLeft: 	y=0.5; 	x=0; break;
		case alignNormal: 	y=0.5;  x=0; break;
		case alignRight: 	y=0.5; 	x=1; break;
		case alignTop: 		y=0; 	x=0.5; break;
		case alignTopLeft: 	y=0; 	x=0; break;
		case alignTopNormal: 	y=0; 	x=0; break;
		case alignTopRight: 	y=0; 	x=1; break;
		default: return;
	}
	
	gtk_misc_set_alignment(GTK_MISC(widget),x,y);
}

void gLabel::setText(const char *st)
{
	gtk_label_set_text(GTK_LABEL(widget),st);
	//fontModified();
}

void gTextLabel::setText(const char *st)
{
	gtk_label_set_markup(GTK_LABEL(widget),st);
	//fontModified();

}

int gLabel::getBorder()
{
	return Frame_getBorder(GTK_FRAME(widget->parent));
}

void gLabel::setBorder(int vl)
{
	Frame_setBorder(GTK_FRAME(widget->parent),vl);
}

/*****************************************************************************

Simple Label

******************************************************************************/
gboolean label_exp(GtkWidget *wid,GdkEventExpose *e,gSimpleLabel *d)
{
	GtkStyle *sty=gtk_widget_get_style(wid);
	GdkGC *gc=gdk_gc_new(wid->window);
	int vw,vh,lw,lh;
	
	if (sty) gdk_gc_set_foreground(gc,&sty->fg[GTK_STATE_NORMAL]);
	
	switch (d->lay_x)
	{
		case 0: pango_layout_set_alignment(d->layout,PANGO_ALIGN_LEFT); break;
		case 1: pango_layout_set_alignment(d->layout,PANGO_ALIGN_CENTER); break;
		case 2: pango_layout_set_alignment(d->layout,PANGO_ALIGN_RIGHT); break;
		case 3: pango_layout_set_alignment(d->layout,PANGO_ALIGN_LEFT); break;
	}
	
	gdk_drawable_get_size (GDK_DRAWABLE(wid->window),&vw,&vh);
	pango_layout_get_size(d->layout,&lw,&lh);
	lw/=PANGO_SCALE;
	lh/=PANGO_SCALE;
	
	switch (d->lay_x)
	{
		case 0: vw=0; break;
		case 1: vw=(vw/2)-(lw/2); break;
		case 2: vw=vw-lw; break;
		case 3: if (gtk_widget_get_default_direction()==GTK_TEXT_DIR_RTL ) vw=vw-lw;
				else vw=0;
				break;
	}
	
	switch (d->lay_y)
	{
		case 0: vh=0; break;
		case 1: vh=(vh/2)-(lh/2); break;
		case 2: vh=vh-lh; break;
	}
	
	gdk_draw_layout(wid->window,gc,vw,vh,d->layout);
	g_object_unref(G_OBJECT(gc));
	return false;
}

gSimpleLabel::gSimpleLabel(gControl *parent) : gControl(parent)
{
	GtkWidget *fr;	

	g_typ=Type_gLabel;
	border=gtk_event_box_new();
	widget=border;
	fr=gtk_frame_new("");
	gtk_frame_set_shadow_type(GTK_FRAME(fr),GTK_SHADOW_NONE);
	gtk_frame_set_label_widget(GTK_FRAME(fr),NULL);
	gtk_container_add (GTK_CONTAINER(border),fr);
	
	layout=gtk_widget_create_pango_layout(border,"");
		
	connectParent();
	initSignals();
	g_signal_connect_after(G_OBJECT(border),"expose-event",G_CALLBACK(label_exp),(gpointer)this);
	setAlignment(alignNormal);
}

gSimpleLabel::~gSimpleLabel()
{
	g_object_unref(G_OBJECT(layout));
}

char *gSimpleLabel::text()
{
	return (char*)pango_layout_get_text(layout);
}

void gSimpleLabel::setText(char *vl)
{

	if (!vl) 
		pango_layout_set_text(layout,"",-1);
	else
		pango_layout_set_text(layout,vl,-1);
		
	if (visible()) gtk_widget_queue_draw_area(border,0,0,width(),height());
}

int gSimpleLabel::getBorder()
{
	return Frame_getBorder(GTK_FRAME(gtk_bin_get_child(GTK_BIN(border))));
}

void gSimpleLabel::setBorder(int vl)
{
	GtkFrame *fr=(GtkFrame*)gtk_bin_get_child(GTK_BIN(border));
	
	Frame_setBorder(GTK_FRAME(fr),vl);
	if (visible()) gtk_widget_queue_draw_area(border,0,0,width(),height());
}

int gSimpleLabel::alignment()
{
	return align;
}

void gSimpleLabel::setAlignment(int al)
{	
	switch (al)
	{
		case alignBottom:       lay_y=2;  lay_x=1; break;
		case alignBottomLeft:   lay_y=2;  lay_x=0; break;
		case alignBottomNormal: lay_y=2;  lay_x=3; break;
		case alignBottomRight:  lay_y=2;  lay_x=2; break;
		case alignCenter:       lay_y=1;  lay_x=1; break;
		case alignLeft:         lay_y=1;  lay_x=0; break;
		case alignNormal:       lay_y=1;  lay_x=3; break;
		case alignRight:        lay_y=1;  lay_x=2; break;
		case alignTop:          lay_y=0;  lay_x=1; break;
		case alignTopLeft:      lay_y=0;  lay_x=0; break;
		case alignTopNormal:    lay_y=0;  lay_x=3; break;
		case alignTopRight:     lay_y=0;  lay_x=2; break;
		default: return;
	}
	
	align=al;
	if (visible()) gtk_widget_queue_draw_area(border,0,0,width(),height());

}

long gSimpleLabel::backGround()
{
	return get_gdk_bg_color(border);
}

void gSimpleLabel::setBackGround(long color)
{
	set_gdk_bg_color(widget,color);	
	if (!border->window) gtk_widget_realize(border);
	gdk_window_process_updates(border->window,true);
	if (visible()) gtk_widget_queue_draw_area(border,0,0,width(),height());
}

long gSimpleLabel::foreGround()
{
	return get_gdk_fg_color(widget);
}

void gSimpleLabel::setForeGround(long color)
{	
	set_gdk_fg_color(widget,color);
	if (!border->window) gtk_widget_realize(border);
	gdk_window_process_updates(border->window,true);
	if (visible()) gtk_widget_queue_draw_area(border,0,0,width(),height());
}

void gSimpleLabel::setFont(gFont *ft)
{
	PangoFontDescription *desc=pango_context_get_font_description(ft->ct);
	
	font_change=true;
	gtk_widget_modify_font(widget,desc);
	pango_layout_set_font_description(layout,desc);
	pango_layout_context_changed(layout);
	if (visible()) gtk_widget_queue_draw_area(border,0,0,width(),height());
}

