/***************************************************************************

  GL.c

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __GL_C

#include "gambas.h"
#include "gb_common.h"
#include "main.h"

#include "GL.h"

#include <GL/gl.h>

#include "GLprimitives.h"
#include "GLcoordTransf.h"
#include "GLmodesExec.h"
#include "GLcolorLighting.h"
#include "GLframeBufferOps.h"
#include "GLrasterization.h"

/**************************************************************************/

BEGIN_PROPERTY(GLINFO_vendor)

	GB.ReturnNewZeroString((char *) glGetString(GL_VENDOR));
	
END_PROPERTY

BEGIN_PROPERTY(GLINFO_renderer)

	GB.ReturnNewZeroString((char *) glGetString(GL_RENDERER));
	
END_PROPERTY

BEGIN_PROPERTY(GLINFO_version)

	GB.ReturnNewZeroString((char *) glGetString(GL_VERSION));
	
END_PROPERTY

BEGIN_PROPERTY(GLINFO_extensions)

	GB.ReturnNewZeroString((char *) glGetString(GL_EXTENSIONS));
	
END_PROPERTY

/**************************************************************************/

GB_DESC Cgl[] =
{
	GB_DECLARE("Gl",0), GB_NOT_CREATABLE(),
	
	/* Infos */
	GB_STATIC_PROPERTY_READ("Vendor", "s", GLINFO_vendor),
	GB_STATIC_PROPERTY_READ("Renderer", "s", GLINFO_renderer),
	GB_STATIC_PROPERTY_READ("Version", "s", GLINFO_version),
	GB_STATIC_PROPERTY_READ("Extensions", "s", GLINFO_extensions),

	/* Primitives - see GLprimitives.h */
	GB_STATIC_PROPERTY("EdgeFlag", 	"b", GLEDGEFLAG),
	GB_STATIC_METHOD("Begin", NULL, GLBEGIN, "(Primitive)i"),
	GB_STATIC_METHOD("End", NULL, GLEND, NULL),
	GB_STATIC_METHOD("Rect", NULL, GLRECT, "(x1)f(y1)f(x2)f(y2)f"),
	GB_STATIC_METHOD("Vertex", NULL, GLVERTEX, "(x)f(y)f[(z)f(w)f]"),
	GB_STATIC_METHOD("VertexArray", NULL, GLVERTEXARRAY, "(Coords)Float[]"),

	/* Coordinate Transformation - see GLcoordTransf.h */
	GB_STATIC_PROPERTY("MatrixMode", "i", GLMATRIXMODE),
	GB_STATIC_METHOD("DepthRange", NULL, GLDEPTHRANGE, "(near)f(far)f"),
	GB_STATIC_METHOD("Frustum", NULL, GLFRUSTUM, "(left)f(right)f(bottom)f(top)f(near)f(far)f"),
	GB_STATIC_METHOD("GetMatrix", "Float[]", GLGETMATRIX, "[(Mode)i]"),	
	GB_STATIC_METHOD("GetViewport", "Integer[]", GLGETVIEWPORT, NULL),
	GB_STATIC_METHOD("LoadIdentity", NULL, GLLOADIDENTITY, NULL),
	GB_STATIC_METHOD("LoadMatrix", NULL, GLLOADMATRIX, "(Matrix)Float[]"),	
	GB_STATIC_METHOD("MultMatrix", NULL, GLMULTMATRIX, "(Matrix)Float[]"),	
	GB_STATIC_METHOD("Ortho", NULL, GLORTHO, "(left)f(right)f(bottom)f(top)f(near)f(far)f"),
	GB_STATIC_METHOD("PopMatrix", NULL, GLPOPMATRIX, NULL),
	GB_STATIC_METHOD("PushMatrix", NULL, GLPUSHMATRIX, NULL),
	GB_STATIC_METHOD("Rotate", NULL, GLROTATE, "(angle)f(x)f(y)f(z)f"),
	GB_STATIC_METHOD("Translate", NULL, GLTRANSLATE, "(x)f(y)f(z)f"),
	GB_STATIC_METHOD("Scale", NULL, GLSCALE, "(x)f(y)f(z)f"),
	GB_STATIC_METHOD("Viewport", NULL, GLVIEWPORT, "(x)i(y)i(width)i(height)i"),
	GB_STATIC_METHOD("ViewportMaxSize", "Integer[]", GLVIEWPORTMAXSIZE, NULL),

	/* Modes and Execution - see GLmodesExec.h */
	GB_STATIC_PROPERTY_SELF("Hint", ".Hint"),
	GB_STATIC_PROPERTY_SELF("Lights", ".Lights"),
	GB_STATIC_METHOD("Disable", NULL, GLDISABLE, "(Capacity)i"),
	GB_STATIC_METHOD("Enable", NULL, GLENABLE, "(Capacity)i"),
	GB_STATIC_METHOD("Flush", NULL, GLFLUSH, NULL),
	GB_STATIC_METHOD("Finish", NULL, GLFINISH, NULL),
	GB_STATIC_METHOD("isEnabled", "b", GLISENABLED, "(Capacity)i"),

	/* Coloring and Lighting - see GLcolorLighting.h */
	GB_STATIC_PROPERTY_SELF("Material", ".Material"),
	GB_STATIC_PROPERTY("Normalize", "b", GLNORMALIZE),
	GB_STATIC_PROPERTY("ShadeModel", "i", GLSHADEMODEL),
	GB_STATIC_METHOD("Color", NULL, GLCOLOR, "(red)f(green)f(blue)f[(alpha)f]"),
	GB_STATIC_METHOD("ColorArray", NULL, GLCOLORARRAY, "(Colors)Float[]"),
	GB_STATIC_METHOD("GetColor", "Float[]", GLGETCOLOR, NULL),
	GB_STATIC_METHOD("GetNormal", "Float[]", GLGETNORMAL, NULL),
	GB_STATIC_METHOD("Normal", NULL, GLNORMAL, "(nx)f(ny)f(nz)f"),

	/* FrameBuffer Operations - see GLframeBufferOps.h  */
	GB_STATIC_PROPERTY("DepthFunc", "i", GLDEPTHFUNC),
	GB_STATIC_PROPERTY("DepthTest", "b", GLDEPTHTEST),
	GB_STATIC_PROPERTY("DepthMask", "b", GLDEPTHMASK),
	GB_STATIC_METHOD("Clear", NULL, GLCLEAR, "(Mask)i"),
	GB_STATIC_METHOD("ClearColor", NULL, GLCLEARCOLOR, "(Red)f(Green)f(Blue)f(Alpha)f"),
	GB_STATIC_METHOD("ClearAccum", NULL, GLCLEARACCUM, "(Red)f(Green)f(Blue)f(Alpha)f"),
	GB_STATIC_METHOD("ClearDepth", NULL, GLCLEARDEPTH, "(Depth)f"),
	GB_STATIC_METHOD("ClearIndex", NULL, GLCLEARINDEX, "(Value)f"),
	GB_STATIC_METHOD("ClearStencil", NULL, GLCLEARSTENCIL, "(Value)i"),

	/* Rasterization - see GLrasterization.h  */
	GB_STATIC_PROPERTY("CullFace", "b", GLCULLFACE),
	GB_STATIC_PROPERTY("LineWidth", "f", GLLINEWIDTH),
	GB_STATIC_PROPERTY("PointSize", "f", GLPOINTSIZE),

	GB_END_DECLARE
};
