/***************************************************************************

  GLcolorLighting.c

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __GLCOLORLIGHTING_C

#include "gambas.h"
#include "gb_common.h"
#include "main.h"

#include <GL/gl.h>

static int currentLight = 0;
static void *hiddenLights;

/**************************************************************************

  Lights management : .....

***************************************************************************/

BEGIN_PROPERTY(GLLIGHT_enabled)

	if (READ_PROPERTY)
		GB.ReturnBoolean(glIsEnabled(GL_LIGHT0+currentLight));
	else
	{
		if (VPROP(GB_BOOLEAN))
			glEnable(GL_LIGHT0+currentLight);
		else
			glDisable(GL_LIGHT0+currentLight);
	}

END_PROPERTY

BEGIN_PROPERTY(GLLIGHT_ambient)
	

	if (READ_PROPERTY)
	{
		GLfloat params[4];
		GB_ARRAY fArray;
		uint i = 0;

		GB.Array.New(&fArray , GB_T_FLOAT , 4);
		glGetLightfv(GL_LIGHT0+currentLight, GL_AMBIENT, params);

		for (i=0;i<4; i++)
			*((double *)GB.Array.Get(fArray, i)) = params[i];
	
		GB.ReturnObject(fArray);
	}
	else
	{
		GLfloat params[4];
		GB_ARRAY fArray = (GB_ARRAY) VPROP(GB_OBJECT);
		uint i=0;
		
		if (GB.Array.Count(fArray)!=4)
			return;
		
		for (i=0; i<4; i++)
			params[i]=*((double *)GB.Array.Get(fArray,i));
		
		glLightfv(GL_LIGHT0+currentLight, GL_AMBIENT, params);
	}
	
END_METHOD

BEGIN_PROPERTY(GLLIGHT_diffuse)
	
	if (READ_PROPERTY)
	{
		GLfloat params[4];
		GB_ARRAY fArray;
		uint i = 0;

		GB.Array.New(&fArray , GB_T_FLOAT , 4);
		glGetLightfv(GL_LIGHT0+currentLight, GL_DIFFUSE, params);

		for (i=0;i<4; i++)
			*((double *)GB.Array.Get(fArray, i)) = params[i];
	
		GB.ReturnObject(fArray);
	}
	else
	{
		GLfloat params[4];
		GB_ARRAY fArray = (GB_ARRAY) VPROP(GB_OBJECT);
		uint i=0;
		
		if (GB.Array.Count(fArray)!=4)
			return;
		
		for (i=0; i<4; i++)
			params[i]=*((double *)GB.Array.Get(fArray,i));
		
		glLightfv(GL_LIGHT0+currentLight, GL_DIFFUSE, params);
	}
	
END_METHOD

BEGIN_PROPERTY(GLLIGHT_specular)
	
	if (READ_PROPERTY)
	{
		GLfloat params[4];
		GB_ARRAY fArray;
		uint i = 0;

		GB.Array.New(&fArray , GB_T_FLOAT , 4);
		glGetLightfv(GL_LIGHT0+currentLight, GL_SPECULAR, params);

		for (i=0;i<4; i++)
			*((double *)GB.Array.Get(fArray, i)) = params[i];
	
		GB.ReturnObject(fArray);
	}
	else
	{
		GLfloat params[4];
		GB_ARRAY fArray = (GB_ARRAY) VPROP(GB_OBJECT);
		uint i=0;
		
		if (GB.Array.Count(fArray)!=4)
			return;
		
		for (i=0; i<4; i++)
			params[i]=*((double *)GB.Array.Get(fArray,i));
		
		glLightfv(GL_LIGHT0+currentLight, GL_SPECULAR, params);
	}
	
END_METHOD

BEGIN_PROPERTY(GLLIGHT_position)
	
	if (READ_PROPERTY)
	{
		GLfloat params[4];
		GB_ARRAY fArray;
		uint i = 0;

		GB.Array.New(&fArray , GB_T_FLOAT , 4);
		glGetLightfv(GL_LIGHT0+currentLight, GL_POSITION, params);

		for (i=0;i<4; i++)
			*((double *)GB.Array.Get(fArray, i)) = params[i];
	
		GB.ReturnObject(fArray);
	}
	else
	{
		GLfloat params[4];
		GB_ARRAY fArray = (GB_ARRAY) VPROP(GB_OBJECT);
		uint i=0;
		
		if (GB.Array.Count(fArray)!=4)
			return;
		
		for (i=0; i<4; i++)
			params[i]=*((double *)GB.Array.Get(fArray,i));
		
		glLightfv(GL_LIGHT0+currentLight, GL_POSITION, params);
	}
	
END_METHOD

BEGIN_PROPERTY(GLLIGHT_spotdirection)
	
	if (READ_PROPERTY)
	{
		GLfloat params[4];
		GB_ARRAY fArray;
		uint i = 0;

		GB.Array.New(&fArray , GB_T_FLOAT , 4);
		glGetLightfv(GL_LIGHT0+currentLight, GL_SPOT_DIRECTION, params);

		for (i=0;i<4; i++)
			*((double *)GB.Array.Get(fArray, i)) = params[i];
	
		GB.ReturnObject(fArray);
	}
	else
	{
		GLfloat params[4];
		GB_ARRAY fArray = (GB_ARRAY) VPROP(GB_OBJECT);
		uint i=0;
		
		if (GB.Array.Count(fArray)!=4)
			return;
		
		for (i=0; i<4; i++)
			params[i]=*((double *)GB.Array.Get(fArray,i));
		
		glLightfv(GL_LIGHT0+currentLight, GL_SPOT_DIRECTION, params);
	}
	
END_METHOD

BEGIN_PROPERTY(GLLIGHT_spotcutoff)
	
	if (READ_PROPERTY)
	{
		GLfloat param[0];

		glGetLightfv(GL_LIGHT0+currentLight, GL_SPOT_CUTOFF, param);
		GB.ReturnFloat(param[0]);
	}
	else
		glLightf(GL_LIGHT0+currentLight, GL_SPOT_CUTOFF, VPROP(GB_FLOAT));
	
END_METHOD

BEGIN_PROPERTY(GLLIGHT_spotexponent)
	
	if (READ_PROPERTY)
	{
		GLfloat param[0];

		glGetLightfv(GL_LIGHT0+currentLight, GL_SPOT_EXPONENT, param);
		GB.ReturnFloat(param[0]);
	}
	else
		glLightf(GL_LIGHT0+currentLight, GL_SPOT_EXPONENT, VPROP(GB_FLOAT));
	
END_METHOD

BEGIN_PROPERTY(GLLIGHT_constantattenuation)
	
	if (READ_PROPERTY)
	{
		GLfloat param[0];

		glGetLightfv(GL_LIGHT0+currentLight, GL_CONSTANT_ATTENUATION, param);
		GB.ReturnFloat(param[0]);
	}
	else
		glLightf(GL_LIGHT0+currentLight, GL_CONSTANT_ATTENUATION, VPROP(GB_FLOAT));
	
END_METHOD

BEGIN_PROPERTY(GLLIGHT_linearattenuation)
	
	if (READ_PROPERTY)
	{
		GLfloat param[0];

		glGetLightfv(GL_LIGHT0+currentLight, GL_LINEAR_ATTENUATION, param);
		GB.ReturnFloat(param[0]);
	}
	else
		glLightf(GL_LIGHT0+currentLight, GL_LINEAR_ATTENUATION, VPROP(GB_FLOAT));
	
END_METHOD

BEGIN_PROPERTY(GLLIGHT_quadraticattenuation)
	
	if (READ_PROPERTY)
	{
		GLfloat param[0];

		glGetLightfv(GL_LIGHT0+currentLight, GL_QUADRATIC_ATTENUATION, param);
		GB.ReturnFloat(param[0]);
	}
	else
		glLightf(GL_LIGHT0+currentLight, GL_QUADRATIC_ATTENUATION, VPROP(GB_FLOAT));
	
END_METHOD

GB_DESC CglLight[] =
{
	GB_DECLARE(".glLight", 0), GB_VIRTUAL_CLASS(),

	GB_PROPERTY("Enabled", "b", GLLIGHT_enabled),
	GB_PROPERTY("Ambient", "Float[]", GLLIGHT_ambient),
	GB_PROPERTY("Diffuse", "Float[]", GLLIGHT_diffuse),
	GB_PROPERTY("Specular", "Float[]", GLLIGHT_specular),
	GB_PROPERTY("Position", "Float[]", GLLIGHT_position),
	GB_PROPERTY("Spot_Direction", "Float[]", GLLIGHT_spotdirection),
	GB_PROPERTY("Spot_CutOff", "f", GLLIGHT_spotcutoff),
	GB_PROPERTY("Spot_Exponent", "f", GLLIGHT_spotexponent),
	GB_PROPERTY("Constant_Attenuation", "f", GLLIGHT_constantattenuation),
	GB_PROPERTY("Linear_Attenuation", "f", GLLIGHT_linearattenuation),
	GB_PROPERTY("Quadratic_Attenuation", "f", GLLIGHT_quadraticattenuation),

	GB_END_DECLARE
};

BEGIN_METHOD_VOID(GLLIGHTS_init)

	GB.New((void **) &hiddenLights, GB.FindClass(".Lights"), NULL, NULL);
	GB.Ref(hiddenLights);

END_METHOD

BEGIN_METHOD_VOID(GLLIGHTS_exit)

	if (hiddenLights)
		GB.Unref((void **) &hiddenLights);
	
END_METHOD

BEGIN_METHOD(GLLIGHTS_get, GB_INTEGER light)

	GLint maxLights = 0;

	currentLight = VARG(light);
	glGetIntegerv(GL_MAX_LIGHTS, &maxLights);

	if (currentLight>maxLights-1)
		return;
	
	GB.ReturnObject(hiddenLights);

END_METHOD

BEGIN_PROPERTY(GLLIGHTS_count)

	GLint param = 0;

	glGetIntegerv(GL_MAX_LIGHTS, &param);
	GB.ReturnInteger(param);

END_PROPERTY

BEGIN_PROPERTY(GLLIGHTS_enabled)

	if (READ_PROPERTY)
		GB.ReturnBoolean(glIsEnabled(GL_LIGHTING));
	else
	{
		if (VPROP(GB_BOOLEAN))
			glEnable(GL_LIGHTING);
		else
			glEnable(GL_LIGHTING);
	}

END_PROPERTY

BEGIN_PROPERTY(GLLIGHTS_ambient)
	
	if (READ_PROPERTY)
	{
		GLfloat params[4];
		GB_ARRAY fArray;
		uint i = 0;

		GB.Array.New(&fArray , GB_T_FLOAT , 4);
		glGetFloatv(GL_LIGHT_MODEL_AMBIENT, params);

		for (i=0;i<4; i++)
			*((double *)GB.Array.Get(fArray, i)) = params[i];
	
		GB.ReturnObject(fArray);
	}
	else
	{
		GLfloat params[4];
		GB_ARRAY fArray = (GB_ARRAY) VPROP(GB_OBJECT);
		uint i=0;
		
		if (GB.Array.Count(fArray)!=4)
			return;
		
		for (i=0; i<4; i++)
			params[i]=*((double *)GB.Array.Get(fArray,i));
		
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, params);
	}
	
END_METHOD

BEGIN_PROPERTY(GLLIGHTS_localviewer)
	
	if (READ_PROPERTY)
	{
		GLfloat param[0];

		glGetFloatv(GL_LIGHT_MODEL_LOCAL_VIEWER, param);
		GB.ReturnFloat(param[0]);
	}
	else
		glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER, VPROP(GB_FLOAT));
	
END_METHOD

BEGIN_PROPERTY(GLLIGHTS_twoside)
	
	if (READ_PROPERTY)
	{
		GLfloat param[0];

		glGetFloatv(GL_LIGHT_MODEL_TWO_SIDE, param);
		GB.ReturnFloat(param[0]);
	}
	else
		glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, VPROP(GB_FLOAT));
	
END_METHOD

GB_DESC CglLights[] =
{
	GB_DECLARE(".Lights", 0), GB_VIRTUAL_CLASS(),
	
	GB_STATIC_METHOD("_init", NULL, GLLIGHTS_init, NULL),
	GB_STATIC_METHOD("_exit", NULL, GLLIGHTS_exit, NULL),
	GB_STATIC_METHOD("_get", ".glLight", GLLIGHTS_get, "(Light)i"),
	GB_STATIC_PROPERTY_READ("Count", "i",  GLLIGHTS_count),
	GB_STATIC_PROPERTY("Enabled", "b", GLLIGHTS_enabled),
	GB_STATIC_PROPERTY("Ambient", "Float[]", GLLIGHTS_ambient),
	GB_STATIC_PROPERTY("Local_Viewer", "f", GLLIGHTS_localviewer),
	GB_STATIC_PROPERTY("Two_Side", "f", GLLIGHTS_twoside),

	GB_END_DECLARE
};

/**************************************************************************

  glMaterial ...

***************************************************************************/

BEGIN_METHOD(GLMATFRONT_ambanddiff, GB_OBJECT array)

	GLfloat color[4];
	GB_ARRAY colors = (GB_ARRAY) VARG(array);
	int i, count = GB.Array.Count(colors);
	color[3] = 1.0;	

	if (count<3 || count>4)
		return;
			
	for (i=0; i<count; i++)
		color[i]=*((double *)GB.Array.Get(colors,i));
	
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color);
	
END_METHOD

GB_DESC CglMaterialFront[] =
{
	GB_DECLARE(".MaterialFront", 0), GB_VIRTUAL_CLASS(),
	GB_STATIC_METHOD("Ambient_And_Diffuse", NULL, GLMATFRONT_ambanddiff, "(Color)Float[]"),
	GB_END_DECLARE
};


GB_DESC CglMaterial[] =
{
	GB_DECLARE(".Material", 0), GB_VIRTUAL_CLASS(),
	GB_STATIC_PROPERTY_SELF("Front", ".MaterialFront"),
//	GB_STATIC_PROPERTY_SELF("Back", ".glMaterialBack"),
//	GB_STATIC_PROPERTY_SELF("FRONT_AND_BACK", ".glMaterialFrontBack"),
	GB_END_DECLARE
};
