/***************************************************************************

  GLcolorLighting.h

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __GLCOLORLIGHTING_H
#define __GLCOLORLIGHTING_H

#include "gambas.h"
#include "gb_common.h"
#include "main.h"

#include <GL/gl.h>

/**************************************************************************/

BEGIN_PROPERTY(GLSHADEMODEL)

	if (READ_PROPERTY)
	{
		GLint param;

		glGetIntegerv(GL_SHADE_MODEL, &param);
		GB.ReturnInteger(param);
	}
	else
		glShadeModel(VPROP(GB_INTEGER));

END_METHOD

BEGIN_METHOD(GLCOLOR, GB_FLOAT red; GB_FLOAT green; GB_FLOAT blue; GB_FLOAT alpha)

	if (MISSING(alpha))
		glColor3f(VARG(red), VARG(green), VARG(blue));
	else
		glColor4f(VARG(red), VARG(green), VARG(blue), VARG(alpha));
		
END_METHOD

BEGIN_METHOD(GLCOLORARRAY, GB_OBJECT array)

	GLfloat r,g,b,a;
	GB_ARRAY color = (GB_ARRAY) VARG(array);
	int count = GB.Array.Count(color);
	
	if (count<3)
		return;

	r = *((double *)GB.Array.Get(color,0));
	g = *((double *)GB.Array.Get(color,1));
	b = *((double *)GB.Array.Get(color,2));
	
	if (count==3)
		glColor3f(r, g, b);
	else
	{
		a = *((double *)GB.Array.Get(color,3));
		glColor4f(r, g, b, a);
	}
	  
END_METHOD

BEGIN_METHOD_VOID(GLGETCOLOR)

	GLfloat params[4];
	GB_ARRAY fArray;
	uint i;

	GB.Array.New(&fArray , GB_T_FLOAT , 4);
	glGetFloatv(GL_CURRENT_COLOR, params);

	for (i=0;i<4; i++)
		*((double *)GB.Array.Get(fArray, i)) = params[i];
	
	GB.ReturnObject(fArray);
	
END_METHOD

BEGIN_METHOD_VOID(GLGETNORMAL)

	GLfloat params[3];
	GB_ARRAY fArray;
	uint i;

	GB.Array.New(&fArray , GB_T_FLOAT , 3);
	glGetFloatv(GL_CURRENT_NORMAL, params);

	for (i=0;i<3; i++)
		*((double *)GB.Array.Get(fArray, i)) = params[i];
	
	GB.ReturnObject(fArray);
	
END_METHOD

	BEGIN_METHOD(GLNORMAL, GB_FLOAT nx; GB_FLOAT ny; GB_FLOAT nz)

	glNormal3f(VARG(nx), VARG(ny), VARG(nz));
		
END_METHOD

BEGIN_PROPERTY(GLNORMALIZE)

	if (READ_PROPERTY)
		GB.ReturnBoolean(glIsEnabled(GL_NORMALIZE));
	else
	{
		if (VPROP(GB_BOOLEAN))
			glEnable(GL_NORMALIZE);
		else
			glDisable(GL_NORMALIZE);
	}

END_PROPERTY

#endif /* __GLCOLORLIGHTING_H */
