/***************************************************************************

  GLprimitives.h

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __GLPRIMITIVES_H
#define __GLPRIMITIVES_H

#include "gambas.h"
#include "gb_common.h"
#include "main.h"

#include <GL/gl.h>

/**************************************************************************/

BEGIN_PROPERTY(GLEDGEFLAG)

	if (READ_PROPERTY)
	{
		GLboolean param = 0;

		glGetBooleanv(GL_EDGE_FLAG, &param);
		GB.ReturnBoolean(param);
	}
	else
		glEdgeFlag(VPROP(GB_BOOLEAN));
  
END_PROPERTY

BEGIN_METHOD(GLBEGIN, GB_INTEGER primitive)

	glBegin(VARG(primitive));
  
END_METHOD

BEGIN_METHOD_VOID(GLEND)

	glEnd();
 
END_METHOD

BEGIN_METHOD(GLRECT, GB_FLOAT x1; GB_FLOAT y1; GB_FLOAT x2; GB_FLOAT y2)

	glRectf(VARG(x1), VARG(y1), VARG(x2), VARG(y2));
  
END_METHOD

BEGIN_METHOD(GLVERTEX, GB_FLOAT x; GB_FLOAT y; GB_FLOAT z; GB_FLOAT w)

	if (MISSING(z))
	{
		glVertex2f(VARG(x), VARG(y));
		return;
	}
	if (MISSING(w))
		glVertex3f(VARG(x), VARG(y), VARG(z));
	else	
		glVertex4f(VARG(x), VARG(y), VARG(z), VARG(w));
  
END_METHOD

BEGIN_METHOD(GLVERTEXARRAY, GB_OBJECT array)

	GLfloat x,y,z,w;
	GB_ARRAY vertex = (GB_ARRAY) VARG(array);
	int count = GB.Array.Count(vertex);
	
	if (count<2)
		return;

	x = *((double *)GB.Array.Get(vertex,0));
	y = *((double *)GB.Array.Get(vertex,1));
	
	if (count==2)
	{
		glVertex2f(x, y);
		return;
	}
	
	z = *((double *)GB.Array.Get(vertex,2));
	
	if (count==3)
		glVertex3f(x, y, z);
	else
	{
		w = *((double *)GB.Array.Get(vertex,3));
		glVertex4f(x, y, z, w);
	}
	  
END_METHOD

#endif /* __GLPRIMITIVES_H */
