/***************************************************************************

  CContainer.cpp

  The Container class

  (c) 2000-2005 Beno� Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CCONTAINER_CPP

#include <qnamespace.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qlayout.h>
#include <qevent.h>

#include "gambas.h"

#include "CWidget.h"
#include "CWindow.h"
#include "CConst.h"

#include "CContainer.h"

// #define DEBUG_ME

static QWidget *get_widget(QObjectList *list)
{
  for(;;)
  {
    QObject *ob = list->current();
    if (!ob)
      return NULL;
    list->next();
    if (ob->isWidgetType())
    {
      if (!((QWidget *)ob)->isHidden() && !((QWidget *)ob)->isA("QSizeGrip"))
        return (QWidget *)ob;
    }
  }
}

#define WIDGET_TYPE QWidget *
#define CONTAINER_TYPE QFrame *
#define ARRANGEMENT_TYPE CCONTAINER_ARRANGEMENT *

#define IS_RIGHT_TO_LEFT() qApp->reverseLayout()

#define GET_WIDGET(_object) ((CWIDGET *)_object)->widget
#define GET_CONTAINER(_object) ((CCONTAINER *)_object)->container
#define GET_ARRANGEMENT(_object) ((CCONTAINER_ARRANGEMENT *)_object)
#define IS_EXPAND(_object) CWIDGET_test_flag(_object, WF_EXPAND)
#define IS_DESIGN(_object) CWIDGET_test_flag(_object, WF_DESIGN)

#define IS_WIDGET_VISIBLE(_widget) (_widget)->isVisible()
#define GET_WIDGET_CONTENTS(_widget, _x, _y, _w, _h) \
	_x = (_widget)->contentsRect().x(); \
	_y = (_widget)->contentsRect().y(); \
	_w = (_widget)->contentsRect().width(); \
	_h = (_widget)->contentsRect().height();
#define GET_WIDGET_X(_widget) (_widget)->x()
#define GET_WIDGET_Y(_widget) (_widget)->y()
#define GET_WIDGET_W(_widget) (_widget)->width()
#define GET_WIDGET_H(_widget) (_widget)->height()
#define MOVE_WIDGET(_widget, _x, _y) (_widget)->move(_x, _y)
#define RESIZE_WIDGET(_widget, _w, _h) (_widget)->resize(_w, _h)
#define MOVE_RESIZE_WIDGET(_widget, _x, _y, _w, _h) (_widget)->setGeometry(_x, _y, _w, _h)

#define INIT_CHECK_CHILDREN_LIST(_widget) \
  QObjectList *list = (QObjectList *)(_widget)->children(); \
  if (!list || list->count() == 0) \
    return;

#define RESET_CHILDREN_LIST() list->first()
#define GET_NEXT_CHILD_WIDGET() get_widget(list)

#define GET_OBJECT_FROM_WIDGET(_widget) CWidget::get(_widget)

#define FUNCTION_NAME CCONTAINER_arrange

#include "gb.form.arrangement.h"



#define arrange_later arrange_now

static void arrange_now(QWidget *cont)
{
  void *_object = CWidget::get(cont);

  if (THIS_ARRANGEMENT->mode == ARRANGE_NONE)
    return;

  //qDebug("now: %p: dirty = TRUE", THIS);
  //THIS_ARRANGEMENT->dirty = TRUE;

  CCONTAINER_arrange(THIS);
}

#if 0
static void post_arrange_later(void *_object)
{
  if (WIDGET && THIS_ARRANGEMENT->dirty)
    CCONTAINER_arrange(THIS);

  GB.Unref(&_object);
}

static void arrange_later(QWidget *cont)
{
  void *_object = CWidget::get(cont);

  if (THIS_ARRANGEMENT->dirty || THIS_ARRANGEMENT->mode == ARRANGE_NONE)
    return;

  GB.Ref(_object);

  qDebug("later: %p: dirty = TRUE", THIS);
  THIS_ARRANGEMENT->dirty = TRUE;

  GB.Post((void (*)())post_arrange_later, (long)THIS);
}
#endif

/***************************************************************************

  class MyContainer

***************************************************************************/

MyContainer::MyContainer(QWidget *parent)
: QFrame(parent)
{
}


void MyContainer::frameChanged(void)
{
  //qDebug("MyContainer::frameChanged %p", CWidget::get(this));
  QFrame::frameChanged();
  //CCONTAINER_arrange(this);
  arrange_later(this);
}

void MyContainer::resizeEvent(QResizeEvent *e)
{
  //qDebug("MyContainer::resizeEvent %p", CWidget::get(this));
  QFrame::resizeEvent(e);
  arrange_later(this);
}

void MyContainer::showEvent(QShowEvent *e)
{
  //qDebug("MyContainer::showEvent %p: %d", CWidget::get(this), this->isVisible());
  QFrame::showEvent(e);
  arrange_now(this);
}

void MyContainer::childEvent(QChildEvent *e)
{
  //qDebug("MyContainer::childEvent %p", CWidget::get(this));
  QFrame::childEvent(e);

  if (!e->child()->isWidgetType())
    return;

  if (e->inserted())
  {
    e->child()->installEventFilter(this);
    //qDebug("inserted");
  }
  else if (e->removed())
  {
    e->child()->removeEventFilter(this);
    //qDebug("removed");
  }

  arrange_later(this);
}

bool MyContainer::eventFilter(QObject *o, QEvent *e)
{
  int type = e->type();

  if (type == QEvent::Move || type == QEvent::Resize || type == QEvent::Show || type == QEvent::Hide || type == EVENT_EXPAND)
  {
    //if (type == QEvent::Resize)
    //  qDebug("MyContainer::eventFilter: Resize");
    arrange_later(this);
  }

  return QObject::eventFilter(o, e);
}


/***************************************************************************

  CContainer

***************************************************************************/

static QRect getRect(QWidget *w)
{
	if (w->inherits("QFrame"))
		return ((QFrame *)w)->contentsRect();
	else
		return QRect(0, 0, w->width(), w->height());
}

BEGIN_METHOD_VOID(CCONTAINER_next)

  #ifdef DEBUG
  if (!THIS_CONTAINER)
    qDebug("Null container");
  #endif

  QObjectList *list = (QObjectList *)THIS_CONTAINER->children();
  unsigned int index;
  CWIDGET *widget;

  for(;;)
  {
    index = ENUM(int);

    if (list == NULL || index >= list->count())
    {
      GB.StopEnum();
      return;
    }

    ENUM(int) = index + 1;

    widget = CWidget::getReal(list->at(index));
    if (widget)
    {
      GB.ReturnObject(widget);
      return;
    }
  }

END_METHOD


BEGIN_PROPERTY(CCONTAINER_count)

  QWidget *wid = THIS_CONTAINER;
  QObjectList *list;
  QObject *ob;
  int n = 0;

  if (!wid)
  	goto __DONE;

	list = (QObjectList *)wid->children();
	if (!list || !list->count())
		goto __DONE;

	list->first();
  for(;;)
  {
    ob = list->current();
    if (!ob)
      break;
    list->next();
    if (ob->isWidgetType() && CWidget::getReal(ob))
    	n++;
  }

__DONE:
  GB.ReturnInteger(n);

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_x)

  #ifdef DEBUG
  if (!THIS_CONTAINER)
    qDebug("Null container");
  #endif

	QRect r = getRect(THIS_CONTAINER);
  QPoint p(r.x(), r.y());

  GB.ReturnInteger(THIS_CONTAINER->mapTo(WIDGET, p).x());

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_y)

  #ifdef DEBUG
  if (!THIS_CONTAINER)
    qDebug("Null container");
  #endif

	QRect r = getRect(THIS_CONTAINER);
  QPoint p(r.x(), r.y());

  GB.ReturnInteger(THIS_CONTAINER->mapTo(WIDGET, p).y());

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_width)

  #ifdef DEBUG
  if (!THIS_CONTAINER)
    qDebug("Null container");
  #endif

  GB.ReturnInteger(getRect(THIS_CONTAINER).width());

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_height)

  #ifdef DEBUG
  if (!THIS_CONTAINER)
    qDebug("Null container");
  #endif

  GB.ReturnInteger(getRect(THIS_CONTAINER).height());

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_arrangement)

  int arr;

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS_ARRANGEMENT->mode);
  else
  {
    arr = VPROP(GB_INTEGER);
    if (arr < 0 || arr > 8)
      return;
    THIS_ARRANGEMENT->mode = arr;
    arrange_later(THIS_CONTAINER);
  }

END_PROPERTY

BEGIN_PROPERTY(CUSERCONTAINER_arrangement)

	CCONTAINER_arrangement(CWidget::get(THIS_CONTAINER), _param);

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_auto_resize)

  if (READ_PROPERTY)
    GB.ReturnBoolean(THIS_ARRANGEMENT->autoresize);
  else
  {
    THIS_ARRANGEMENT->autoresize = VPROP(GB_BOOLEAN);
    arrange_later(THIS_CONTAINER);
  }

END_PROPERTY

BEGIN_PROPERTY(CUSERCONTAINER_auto_resize)

	CCONTAINER_auto_resize(CWidget::get(THIS_CONTAINER), _param);

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_padding)

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS_ARRANGEMENT->padding);
  else
  {
    int val = VPROP(GB_INTEGER);

    if (val >= 0 && val < 32768)
    {
      THIS_ARRANGEMENT->padding = val;
      arrange_later(THIS_CONTAINER);
    }
  }

END_PROPERTY

BEGIN_PROPERTY(CUSERCONTAINER_padding)

	CCONTAINER_padding(CWidget::get(THIS_CONTAINER), _param);

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_spacing)

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS_ARRANGEMENT->spacing);
  else
  {
    int val = VPROP(GB_INTEGER);

    if (val >= 0 && val < 32768)
    {
      THIS_ARRANGEMENT->spacing = val;
      arrange_later(THIS_CONTAINER);
    }
  }

END_PROPERTY

BEGIN_PROPERTY(CUSERCONTAINER_spacing)

	CCONTAINER_spacing(CWidget::get(THIS_CONTAINER), _param);

END_PROPERTY


BEGIN_METHOD(CUSERCONTROL_new, GB_OBJECT parent)

  MyContainer *wid = new MyContainer(CONTAINER(VARG(parent)));

  CWIDGET_new(wid, (void *)_object);

  THIS->container = wid;
  THIS_ARRANGEMENT->mode = ARRANGE_FILL;
  THIS_ARRANGEMENT->user = true;

  wid->show();

END_METHOD


BEGIN_PROPERTY(CUSERCONTROL_container)

	CCONTAINER *current = (CCONTAINER *)CWidget::get(THIS_CONTAINER);

	if (READ_PROPERTY)
		GB.ReturnObject(current);
	else
	{
		CCONTAINER *cont = (CCONTAINER *)VPROP(GB_OBJECT);
		QWidget *w;
		QWidget *p;

		// sanity checks

		if (!cont)
		{
			THIS->container = WIDGET;
			return;
		}

		if (GB.CheckObject(cont))
			return;

		w = cont->container;
		if (w == THIS->container)
			return;

		for (p = w; p; p = p->parentWidget())
		{
			if (p == WIDGET)
				break;
		}

		if (!p)
			GB.Error("Container must be a child control");
		else
		{
			THIS->container = w;

			CWIDGET_update_design((CWIDGET *)THIS);
			CCONTAINER_arrange(THIS);
		}
	}

END_PROPERTY


BEGIN_PROPERTY(CUSERCONTAINER_container)

	CCONTAINER *before;
	CCONTAINER *after;

	if (READ_PROPERTY)
		CUSERCONTROL_container(_object, _param);
	else
	{
		before = (CCONTAINER *)CWidget::get(THIS->container);

		CUSERCONTROL_container(_object, _param);

		after = (CCONTAINER *)CWidget::get(THIS->container);
		if (before != THIS)
			after->arrangement = before->arrangement;
		CCONTAINER_arrange(after);
	}

END_PROPERTY


GB_DESC CChildrenDesc[] =
{
  GB_DECLARE(".ContainerChildren", sizeof(CCONTAINER)), GB_VIRTUAL_CLASS(),

  GB_METHOD("_next", "Control", CCONTAINER_next, NULL),
  GB_PROPERTY_READ("Count", "i", CCONTAINER_count),

  GB_END_DECLARE
};


GB_DESC CContainerDesc[] =
{
  GB_DECLARE("Container", sizeof(CCONTAINER)), GB_INHERITS("Control"),
  GB_NOT_CREATABLE(),

  GB_PROPERTY_SELF("Children", ".ContainerChildren"),

  GB_PROPERTY_READ("ClientX", "i", CCONTAINER_x),
  GB_PROPERTY_READ("ClientY", "i", CCONTAINER_y),
  GB_PROPERTY_READ("ClientW", "i", CCONTAINER_width),
  GB_PROPERTY_READ("ClientWidth", "i", CCONTAINER_width),
  GB_PROPERTY_READ("ClientH", "i", CCONTAINER_height),
  GB_PROPERTY_READ("ClientHeight", "i", CCONTAINER_height),

  GB_END_DECLARE
};


GB_DESC CUserControlDesc[] =
{
  GB_DECLARE("UserControl", sizeof(CCONTAINER_ARRANGEMENT)), GB_INHERITS("Container"),

  GB_METHOD("_new", NULL, CUSERCONTROL_new, "(Parent)Container;"),

  GB_PROPERTY("Container", "Container", CUSERCONTROL_container),

  GB_CONSTANT("_Properties", "s", CWIDGET_PROPERTIES),

  GB_END_DECLARE
};


GB_DESC CUserContainerDesc[] =
{
  GB_DECLARE("UserContainer", sizeof(CCONTAINER_ARRANGEMENT)), GB_INHERITS("Container"),

  GB_METHOD("_new", NULL, CUSERCONTROL_new, "(Parent)Container;"),

  GB_PROPERTY("Container", "Container", CUSERCONTAINER_container),

  GB_PROPERTY("Arrangement", "i<Arrange>", CUSERCONTAINER_arrangement),
  GB_PROPERTY("AutoResize", "b", CUSERCONTAINER_auto_resize),
  GB_PROPERTY("Padding", "i", CUSERCONTAINER_padding),
  GB_PROPERTY("Spacing", "i", CUSERCONTAINER_spacing),

  GB_CONSTANT("_Properties", "s", "*," CARRANGEMENT_PROPERTIES),
  GB_CONSTANT("_Arrangement", "i", ARRANGE_FILL),

  GB_END_DECLARE
};




