/***************************************************************************

  CWatcher.cpp

  (c) 2000-2005 Benoît Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CWATCHER_CPP

#include <qevent.h>

#include "main.h"
#include "CWatcher.h"

DECLARE_EVENT(EVENT_Move);
DECLARE_EVENT(EVENT_Resize);
DECLARE_EVENT(EVENT_Show);
DECLARE_EVENT(EVENT_Hide);

/** CWatcher class ********************************************************/

CWatcher::CWatcher(CWATCHER *w, CWIDGET *o)
{
	watcher = w;
	control = o;
	control->widget->installEventFilter(this);
}

CWatcher::~CWatcher()
{
	if (control->widget)
		control->widget->removeEventFilter(this);
}

bool CWatcher::eventFilter(QObject* o, QEvent *e)
{
	if (e->type() == QEvent::Move)
		GB.Raise(watcher, EVENT_Move, 0);
	else if (e->type() == QEvent::Resize)
		GB.Raise(watcher, EVENT_Resize, 0);
	else if (e->type() == QEvent::Show)
		GB.Raise(watcher, EVENT_Show, 0);
	else if (e->type() == QEvent::Hide)
		GB.Raise(watcher, EVENT_Hide, 0);
	/*else if (e->type() == QEvent::ChildInserted)
		GB.Raise(watcher, EVENT_ChildInsert, 0);
	else if (e->type() == QEvent::ChildRemoved)
		GB.Raise(watcher, EVENT_ChildRemoved, 0);*/
	return false;
}

/** Watcher class *********************************************************/

BEGIN_METHOD(CWATCHER_new, GB_OBJECT control)

	CWIDGET *control = (CWIDGET *)VARG(control);

	if (GB.CheckObject(control))
		return;

	THIS->watcher = new CWatcher(THIS, control);

	// No need to reference control, as it is already referenced as event observer!

END_METHOD

BEGIN_METHOD_VOID(CWATCHER_free)

	delete THIS->watcher;
	THIS->watcher = 0;

END_METHOD

GB_DESC CWatcherDesc[] =
{
  GB_DECLARE("Watcher", sizeof(CWATCHER)),

  GB_METHOD("_new", NULL, CWATCHER_new, "(Control)Control;"),
  GB_METHOD("_free", NULL, CWATCHER_free, NULL),

  GB_EVENT("Move", NULL, NULL, &EVENT_Move),
  GB_EVENT("Resize", NULL, NULL, &EVENT_Resize),
  GB_EVENT("Show", "b", NULL, &EVENT_Show),
  GB_EVENT("Hide", "b", NULL, &EVENT_Hide),

  GB_CONSTANT("_DefaultEvent", "s", "Resize"),

  GB_END_DECLARE
};
