/***************************************************************************

  event.c

  The SDL component

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benoît Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __EVENT_C

#include "main.h"
#include "gambas.h"
#include "event.h"

#include "SDL.h"


void CloseEvent(void);
void CloseEvent(void);
void ResizeEvent(SDL_ResizeEvent *event);
void AppFocusEvent(SDL_ActiveEvent *event);
void MouseFocusEvent(SDL_ActiveEvent *event);
void MouseButtonEvent(SDL_Event *event);
void MouseMotionDownEvent(SDL_MouseMotionEvent *event);
void KeyPressedEvent(void);
void KeyReleasedEvent(void);

void manageEvent()
{
	SDL_Event event;
	
	while (SDL_PollEvent(&event))
	{
		switch(event.type)
			{
				case SDL_QUIT:
					CloseEvent(); 
					break;
				case SDL_ACTIVEEVENT:
					if (event.active.state==SDL_APPINPUTFOCUS)
						AppFocusEvent(&event.active);
					if (event.active.state==SDL_APPMOUSEFOCUS)
						MouseFocusEvent(&event.active);
					break;
				case SDL_VIDEORESIZE:
					ResizeEvent(&event.resize);
					break;					
				case SDL_MOUSEBUTTONDOWN:
				case SDL_MOUSEBUTTONUP:
					MouseButtonEvent(&event);
					break;					
				case SDL_MOUSEMOTION:
					motionEvent = &event.motion;
					MouseMotionDownEvent(&event.motion);
					break;
				case SDL_KEYDOWN:
					keyevent = &event.key;
					KeyPressedEvent();
					break;
				case SDL_KEYUP:
					keyevent = &event.key;
					KeyReleasedEvent();
					break;
				default:
					break;
				}
	}
}

/* the close event (windowclose or sigterm) */

void CloseEvent()
{
	if (GB.CanRaise(mScreen->hiddenObject, EVENT_Close))
	{
		if (!GB.Raise(mScreen->hiddenObject, EVENT_Close, 0))
			mScreen->WantQuit=1;
	}
	else
    	mScreen->WantQuit=1;
}

/* Resize event */

void ResizeEvent(SDL_ResizeEvent *event)
{
	mScreen->width = event->w;
	mScreen->height = event->h;

  MainScreen = SDL_SetVideoMode (mScreen->width, mScreen->height, MainScreen->format->BitsPerPixel, mScreen->flags| mScreen->border);
  
  if (!MainScreen)
  {
    GB.Error ("Cannot resize the Screen : &1",SDL_GetError());
    return;
  }
  
	if (GB.CanRaise(mScreen->hiddenObject, EVENT_Resize))
		GB.Raise(mScreen->hiddenObject, EVENT_Resize, 0);
}

/* the window/application focus event */

void AppFocusEvent(SDL_ActiveEvent *event)
{
	if (event->gain)
	{
 		if (GB.CanRaise(mScreen->hiddenObject, EVENT_GotFocus))
			GB.Raise(mScreen->hiddenObject, EVENT_GotFocus, 0);
	}
	else
	{	
 		if (GB.CanRaise(mScreen->hiddenObject, EVENT_LostFocus))
			GB.Raise(mScreen->hiddenObject, EVENT_LostFocus, 0);
	}
}

/* the mouse focus event */

void MouseFocusEvent(SDL_ActiveEvent *event)
{
	if (event->gain)
	{
 		if (GB.CanRaise(mScreen->hiddenObject, EVENT_Enter))
			GB.Raise(mScreen->hiddenObject, EVENT_Enter, 0);
	}
	else
	{	
 		if (GB.CanRaise(mScreen->hiddenObject, EVENT_Leave))
			GB.Raise(mScreen->hiddenObject, EVENT_Leave, 0);
	}
}

/* mouse button events */

void MouseButtonEvent(SDL_Event *event)
{
	if (event->type==SDL_MOUSEBUTTONDOWN)
	{
 		if (GB.CanRaise(mScreen->hiddenObject, EVENT_MouseDown))
			GB.Raise(mScreen->hiddenObject, EVENT_MouseDown, 0);
	}
	else
	{	
 		if (GB.CanRaise(mScreen->hiddenObject, EVENT_MouseUp))
			GB.Raise(mScreen->hiddenObject, EVENT_MouseUp, 0);
	}
}

/* mouse motion down event */

void MouseMotionDownEvent(SDL_MouseMotionEvent *event)
{
	if (!event->state)
		return;
	
	if (GB.CanRaise(mScreen->hiddenObject, EVENT_MouseMove))
		GB.Raise(mScreen->hiddenObject, EVENT_MouseMove, 0);
}

/* key pressed event */

void KeyPressedEvent()
{
	if (GB.CanRaise(mScreen->hiddenObject, EVENT_KeyPressed))
		GB.Raise(mScreen->hiddenObject, EVENT_KeyPressed, 0);
}

/* key released event */

void KeyReleasedEvent()
{
	if (GB.CanRaise(mScreen->hiddenObject, EVENT_KeyReleased))
		GB.Raise(mScreen->hiddenObject, EVENT_KeyReleased, 0);
}
