/***************************************************************************

  screen.c

  The SDL component

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@users.sourceforge.net>
                Bodard Fabien   <gambasfr@wanadoo.fr>
                Daniel Campos Fern�dez <danielcampos@netcourrier.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __SCREEN_C

#include "main.h"
#include <string.h> /* memset */

#include "screen.h"
#include "driver.h"

#include "SDL.h"
#include "SDL_opengl.h"
//#include "SDL_framerate.h"
#include "SDL_syswm.h"

//static FPSmanager fpsm;
static Uint32 start = 0;
static double fcurrRate = 0;
static int fcount = 1;
static int Rate = 0;

void refresh()
{
  if (!MainScreen)
  {
    GB.ReturnInteger(0);
    return;
  }
  
  if (start==0)
    start = SDL_GetTicks();

  SDL_GL_SwapBuffers();
  
/*  if (Rate)
    SDL_framerateDelay(&fpsm);
*/    
  if (fcount==100)
  {
    double value = (SDL_GetTicks() - start)/100;
    
    if (value>0)
      fcurrRate = 1000/value;
    
    fcount = 0;
    start = SDL_GetTicks();
  }
  else
   fcount++;
}

/**************************************************************************

  Screen - class for managing the main screen

***************************************************************************/

BEGIN_METHOD_VOID(SCREEN_init)

//  SDL_initFramerate(&fpsm);

  /* Here we define default datas about the screen */
  GB.Alloc ((void **) &mScreen, sizeof (Mscreen));
  memset (mScreen, 0, sizeof (Mscreen));
 
  /* Default values */
  mScreen->flags = SDL_OPENGL;
  mScreen->Wname = GB.Application.Name();
  mScreen->Gred = mScreen->Ggreen = mScreen->Gblue = 1;
  if (!DriverInfo->wm_available)
    mScreen->fullscreen = 1;
  
END_METHOD

BEGIN_METHOD_VOID(SCREEN_exit)
  
  if (mScreen->hiddenObject)
    GB.Unref((void **) &mScreen->hiddenObject);
  
  GB.Free ((void **) &mScreen);

END_METHOD

BEGIN_METHOD(SCREEN_show, GB_INTEGER width; GB_INTEGER height; GB_INTEGER depth)

  Uint8 depth;

  mScreen->width  = VARG(width);
  mScreen->height = VARG(height);
  depth  = VARG(depth);

  if (mScreen->fullscreen)
    mScreen->flags |= SDL_FULLSCREEN;
/*
  SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16);
*/  
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

  MainScreen = SDL_SetVideoMode (mScreen->width, mScreen->height, depth, mScreen->flags | mScreen->border);
  
  if (!MainScreen)
  {
    GB.Error ("Cannot open the Screen : &1",SDL_GetError());
    return;
  }
  
  SDL_WM_SetCaption(mScreen->Wname, mScreen->Wname);
  
  if (mScreen->fullscreen)
    SDL_SetGamma (mScreen->Gred, mScreen->Ggreen, mScreen->Gblue);
  
  mScreen->WantQuit = 0;
  mScreen->RetVal = 0;

  if (mScreen->hiddenObject)
    return;
  
  GB.New((void **) &mScreen->hiddenObject, GB.FindClass("Screen"), "Screen", 0);
  GB.Ref(mScreen->hiddenObject);
  
  
END_METHOD

BEGIN_METHOD(SCREEN_save, GB_STRING file)

  if (!MainScreen)
    return;
    
  if (SDL_SaveBMP(MainScreen, STRING(file))<0)
    GB.Error(" Cannot Save : &1",SDL_GetError());
  
END_METHOD

BEGIN_METHOD(SCREEN_close, GB_INTEGER RetVal)

  mScreen->WantQuit = 1;
  mScreen->RetVal   = VARGOPT(RetVal,0);

END_METHOD

BEGIN_METHOD_VOID(SCREEN_refresh)

    refresh();
  
END_METHOD

BEGIN_METHOD(SCREEN_gamma, GB_FLOAT red; GB_FLOAT green; GB_FLOAT blue)

  mScreen->Gred   = VARG(red);
  mScreen->Ggreen = VARG(green);
  mScreen->Gblue  = VARG(blue);

  if (!MainScreen)
    return;

  if (MainScreen->flags & SDL_FULLSCREEN)
    SDL_SetGamma (mScreen->Gred, mScreen->Ggreen, mScreen->Gblue);

END_METHOD

BEGIN_METHOD(SCREEN_clear, GB_FLOAT Red; GB_FLOAT Green; GB_FLOAT Blue; GB_FLOAT Alpha)

  if (!MainScreen)
    return;

  /* Clear the color and depth buffers. */
  glClearColor(VARGOPT(Red,0), VARGOPT(Green,0), VARGOPT(Blue,0), VARGOPT(Alpha,0));
  glClear(GL_COLOR_BUFFER_BIT);

END_METHOD

BEGIN_PROPERTY(SCREEN_caption)

  if (READ_PROPERTY)
  {
    GB.ReturnNewZeroString((char *) mScreen->Wname);
    return;
  }
  else
  {
    mScreen->Wname = GB.ToZeroString(PROP(GB_STRING));
    if (MainScreen)
      SDL_WM_SetCaption(mScreen->Wname, mScreen->Wname);
  }

END_PROPERTY

BEGIN_PROPERTY(SCREEN_fullscreen)

  if (READ_PROPERTY)
    GB.ReturnBoolean((int ) mScreen->fullscreen);
  else
  {
    if (DriverInfo->wm_available)
    {
      if ((VPROP(GB_BOOLEAN) && !(mScreen->fullscreen)) ||
         (!VPROP(GB_BOOLEAN) && (mScreen->fullscreen)))
      {
        if (MainScreen)
          SDL_WM_ToggleFullScreen(MainScreen);

	mScreen->fullscreen = !mScreen->fullscreen;
      }
    }
    /* Set the gamma only if fullscreen */
    if (MainScreen)
    {
      if (mScreen->fullscreen)
        SDL_SetGamma (mScreen->Gred, mScreen->Ggreen, mScreen->Gblue);
      else
        SDL_SetGamma (1,1,1);
    }
  }

END_PROPERTY

BEGIN_PROPERTY(SCREEN_border)

  if (READ_PROPERTY)
    GB.ReturnInteger(mScreen->border);
  else
  {
    if (mScreen->border==VPROP(GB_INTEGER))
	  return;

    if ((VPROP(GB_INTEGER)!=SDL_RESIZABLE) && (VPROP(GB_INTEGER)!=0))
	  return;
	
	if (!MainScreen)
	  return;
	
	mScreen->border = VPROP(GB_INTEGER);
    MainScreen = SDL_SetVideoMode (mScreen->width, mScreen->height, MainScreen->format->BitsPerPixel, mScreen->flags | mScreen->border);
  
    if (!MainScreen)
    {
      GB.Error ("Cannot change border the Screen : &1",SDL_GetError());
      return;
    }
  }

END_PROPERTY

BEGIN_PROPERTY(SCREEN_width)

  GB.ReturnInteger(mScreen->width);

END_PROPERTY

BEGIN_PROPERTY(SCREEN_height)

  GB.ReturnInteger(mScreen->height);

END_PROPERTY

BEGIN_PROPERTY(SCREEN_depth)

  if (MainScreen)
    GB.ReturnInteger((Uint8 ) MainScreen->format->BitsPerPixel);
  else
    GB.ReturnInteger(0);

END_PROPERTY

BEGIN_PROPERTY(SCREEN_showed)

  if (MainScreen)
    GB.ReturnBoolean(1);
  else
    GB.ReturnBoolean(0);

END_PROPERTY

BEGIN_METHOD(SCREEN_setframerate, GB_INTEGER rate)

  Rate = VARG(rate);
  
  if (!Rate)
    return;
  
//  SDL_setFramerate(&fpsm, Rate);
    
END_METHOD

BEGIN_PROPERTY(SCREEN_getFPS)

  GB.ReturnInteger(fcurrRate);

END_PROPERTY

BEGIN_PROPERTY(SCREEN_id)

  if (MainScreen)
  {
	SDL_SysWMinfo info;
	SDL_VERSION(&info.version);
	SDL_GetWMInfo(&info);
    GB.ReturnInteger(info.info.x11.window);
  }
  else
    GB.ReturnInteger(0);
  
END_PROPERTY

/**************************************************************************/

GB_DESC Cscreen[] =
{
  GB_DECLARE("Screen", sizeof(CSCREEN)), GB_NOT_CREATABLE(),

  GB_STATIC_METHOD("_init", NULL, SCREEN_init, NULL),
  GB_STATIC_METHOD("_exit", NULL, SCREEN_exit, NULL),

  GB_STATIC_METHOD("Show",  NULL, SCREEN_show,  "(Width)i(Height)i(Depth)i"),
  GB_STATIC_METHOD("Close", NULL, SCREEN_close, "[(return)i]"),
  
  GB_STATIC_METHOD("Save",  NULL,      SCREEN_save, "(file)s"),

  GB_STATIC_METHOD("Clear", NULL, SCREEN_clear, "[(red)f(green)f(blue)f(alpha)f]"),
  GB_STATIC_METHOD("Cls",   NULL, SCREEN_clear, "[(red)f(green)f(blue)f(alpha)f]"),

  GB_STATIC_METHOD("Refresh", NULL, SCREEN_refresh, NULL),
  GB_STATIC_METHOD("Update",  NULL, SCREEN_refresh, NULL),
  
  GB_STATIC_METHOD("SetGamma", NULL, SCREEN_gamma, "(red)f(green)f(blue)f"),

  GB_STATIC_PROPERTY("Caption", "s", SCREEN_caption),
  GB_STATIC_PROPERTY("Text",    "s", SCREEN_caption),
  GB_STATIC_PROPERTY("Title",   "s", SCREEN_caption),
  GB_STATIC_PROPERTY("Border", "i", SCREEN_border),

  GB_STATIC_PROPERTY("Fullscreen", "b", SCREEN_fullscreen),
  GB_STATIC_PROPERTY("Border", "i", SCREEN_border),

  GB_STATIC_PROPERTY_READ("Width",  "i", SCREEN_width),
  GB_STATIC_PROPERTY_READ("W",      "i", SCREEN_width),
  GB_STATIC_PROPERTY_READ("Height", "i", SCREEN_height),
  GB_STATIC_PROPERTY_READ("H"     , "i", SCREEN_height),
  GB_STATIC_PROPERTY_READ("Depth",  "i", SCREEN_depth),
  GB_STATIC_PROPERTY_READ("D",      "i", SCREEN_depth),
  GB_STATIC_PROPERTY_READ("Showed", "b", SCREEN_showed),
  GB_STATIC_PROPERTY_READ("Id",     "i", SCREEN_id),
  GB_STATIC_PROPERTY_READ("Handle", "i", SCREEN_id),

  GB_STATIC_METHOD("SetFramerate", NULL, SCREEN_setframerate, "(rate)i"),
  GB_STATIC_PROPERTY_READ("GetFramerate", "i", SCREEN_getFPS),
	
  GB_EVENT("Refresh", "b", NULL, &EVENT_Refresh),
  GB_EVENT("Close", "b", NULL, &EVENT_Close),
  GB_EVENT("GotFocus", NULL, NULL, &EVENT_GotFocus),
  GB_EVENT("LostFocus", NULL, NULL, &EVENT_LostFocus),
  GB_EVENT("Resize", NULL, NULL, &EVENT_Resize),
  GB_EVENT("Enter", NULL, NULL, &EVENT_Enter),
  GB_EVENT("Leave", NULL, NULL, &EVENT_Leave),
  GB_EVENT("MouseDown", NULL, NULL, &EVENT_MouseDown),
  GB_EVENT("MouseUp", NULL, NULL, &EVENT_MouseUp),
  GB_EVENT("MouseMove", NULL, NULL, &EVENT_MouseMove),
  GB_EVENT("KeyPressed", NULL, NULL, &EVENT_KeyPressed),
  GB_EVENT("KeyReleased", NULL, NULL, &EVENT_KeyReleased),
  
  GB_CONSTANT("Fixed", 		"i", 0),
  GB_CONSTANT("Resizable", 	"i", SDL_RESIZABLE),

  GB_END_DECLARE
};
