/***************************************************************************

  main.c

  The SDL component

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __MAIN_C

#include "main.h"
#include "gb_common.h"
#include "event.h"

#include "driver.h"
#include "surface.h"
#include "screen.h"
#include "color.h"
#include "palette.h"
#include "draw.h"
#include "key.h"
#include "joystick.h"
#include "mouse.h"
#include "font.h"
#include "timer.h"
#include "game.h"

#include "../gb.sdl.h"

#include "SDL.h"
#include "SDL_ttf.h"

Mscreen *mScreen = NULL;
int InEventLoop = 0;
int havePost =0;

GB_INTERFACE GB;
SDL_INTERFACE SDL;

static int my_loop()
{
	InEventLoop++;
	
	if (!MainScreen)
		return (0);

	while (!mScreen->WantQuit)
	{
		manageEvent();
		
		/* Screen_Refresh() event */
		if (GB.CanRaise(mScreen->hiddenObject, EVENT_Refresh))
		{
			if (!GB.Raise(mScreen->hiddenObject, EVENT_Refresh, 0))
				refresh();
		}
		
		/* Posts events */
		if (havePost)
		{
			GB.CheckPost();
			havePost = 0;
		}

		SDL_Delay(1);
	}

	return (mScreen->RetVal);
}

static void my_post()
{
	havePost++;
}

GB_DESC *GB_CLASSES[] =
{
  Cdriver,
  Cscreen,
  Cpalette, Cpalcolor,
  Cfont,  /* Must be defined before drawing (need for init) */
  Cdrawing, Ccolor,
  Csurface, Csurfacealpha, Csurfacecolor,
  Ckey, Ckeycode,
  Cqueryjoy,
  Cmouse,
  Ctimer,
  Cpoke,

  NULL
};

int GB_INIT(void)
{
  GB.GetInterface("gb.sdl", SDL_INTERFACE_VERSION, &SDL);

  if (SDL.Init (SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_JOYSTICK | SDL_INIT_EVENTTHREAD))
    return FALSE;

  if (TTF_Init()==-1)
  {
    GB.Error("TTF_Init : &1\n", TTF_GetError());
    return FALSE;
  }

  SDL_EnableUNICODE(1);

  GB.Hook(GB_HOOK_LOOP, (void *)my_loop);
  GB.Hook(GB_HOOK_POST, (void *)my_post);

  return FALSE;
}

void GB_EXIT(void)
{
  TTF_Quit();
  SDL.Quit();
}
